package com.platomix.activity.clickhouse.model;

import java.io.Serializable;
import java.util.Date;

import com.baomidou.mybatisplus.annotation.IdType;
import com.baomidou.mybatisplus.annotation.TableField;
import com.baomidou.mybatisplus.annotation.TableId;
import com.baomidou.mybatisplus.annotation.TableName;
import com.platomix.activity.enums.ActivityTypeEnum;

import lombok.Data;

/**
 * 
 * Desc:活动会员兑奖记录表
 * <p>
 * Modify:
 * <p>
 * Author: William Lynn
 * <p>
 * Create Date：Sep 2, 2021 4:49:38 PM
 *
 */
@Data
@TableName("BLADE_ACTIVITY_MEMBER_EXCHANGE_RECORD")
public class ActivityMemberExchangeRecord implements Serializable {

	private static final long serialVersionUID = 1L;

	/**
	 * 主键ID
	 */
	@TableId(value = "id", type = IdType.AUTO)
	private Long id;

	/**
	 * 活动UUID
	 */
	@TableField(value = "activity_uuid")
	private String activityUuid;

	/**
	 * 活动名称
	 */
	@TableField(value = "activity_name")
	private String activityName;

	/**
	 * 活动类型 1=活动类 2=订购类 3=游戏类 4=秒杀类 5=助力类 6=抽奖类
	 */
	@TableField(value = "activity_type")
	private Integer activityType;

	/**
	 * 奖品id
	 */
	@TableField(value = "prize_id")
	private String prizeId;

	/**
	 * 奖品名称
	 */
	@TableField(value = "prize_name")
	private String prizeName;

	/**
	 * 中奖码
	 */
	@TableField(value = "win_code")
	private String winCode;

	/**
	 * 奖品等级
	 */
	@TableField(value = "prize_level")
	private Integer prizeLevel;

	/**
	 * 奖品类型 （待定 是否按照兑奖类型来使用该字段）兑奖方式 1=线下邮寄 2=获取卡密 3=手机号码人工导出 4=QQ号人工导出 5=微信账号人工导出
	 * 6=其它
	 */
	@TableField(value = "prize_type")
	private Integer prizeType;

	/**
	 * 兑奖状态 1 已兑 2 未兑
	 */
	@TableField(value = "exchange_status")
	private Integer exchangeStatus;

	/**
	 * 收货电话号码
	 */
	@TableField(value = "charge_phone")
	private String chargePhone;

	/**
	 * 收货人
	 */
	@TableField(value = "consignee")
	private String consignee;

	/**
	 * 配送地址
	 */
	@TableField(value = "shipping_address")
	private String shippingAddress;

	/**
	 * 兑奖时间
	 */
	@TableField(value = "exchange_time")
	private Date exchangeTime;

	/**
	 * 快递单号
	 */
	@TableField(value = "courier_number")
	private String courierNumber;

	@TableField(value = "member_phone")
	private String memberPhone;

	public ActivityDataRecordExchange convert() {
		ActivityDataRecordExchange data = new ActivityDataRecordExchange();
		data.setExchangeId(id);
		data.setActivityUuid(activityUuid);
		data.setActivityName(activityName);
		data.setActivityType(activityType);
		data.setActivityTypeName(ActivityTypeEnum.reslove(activityType).getMessage());
		data.setPrizeId(prizeId);
		data.setPrizeName(prizeName);
		data.setPrizeType(prizeType);
		data.setPrizeLevel(prizeLevel);
		data.setWinCode(winCode);
		data.setPhone(memberPhone);
		data.setExchangeTime(exchangeTime);
		data.setExchangeStatus(exchangeStatus);
		data.setExchangeStatusName(exchangeStatus == 1 ? "已兑奖" : "未兑奖");
		data.setCreateTime(new Date());
		return data;
	}
}
