package com.platomix.activity.clickhouse.model;

import java.io.Serializable;
import java.util.Date;

import com.baomidou.mybatisplus.annotation.IdType;
import com.baomidou.mybatisplus.annotation.TableField;
import com.baomidou.mybatisplus.annotation.TableId;
import com.baomidou.mybatisplus.annotation.TableName;
import com.platomix.activity.enums.ActivityTypeEnum;

import lombok.Data;

/**
 * 
 * Desc: 活动参与记录表
 * <p>
 * Modify:
 * <p>
 * Author: William Lynn
 * <p>
 * Create Date：Sep 2, 2021 4:49:18 PM
 *
 */
@Data
@TableName("BLADE_ACTIVITY_PARTICIPATE_RECORD")
public class ActivityParticipateRecord implements Serializable {

	private static final long serialVersionUID = 1L;

	/**
	 * 主键ID
	 */
	@TableId(value = "id", type = IdType.AUTO)
	private Long id;

	/**
	 * 活动UUID
	 */
	@TableField(value = "activity_uuid")
	private String activityUuid;

	/**
	 * 活动名称
	 */
	@TableField(value = "activity_name")
	private String activityName;

	/**
	 * 活动类型 1=活动类 2=订购类 3=游戏类 4=秒杀类 5=助力类 6=抽奖类
	 */
	@TableField(value = "activity_type")
	private Integer activityType;

	/**
	 * 参与活动手机号
	 */
	@TableField(value = "participate_phone")
	private String participatePhone;

	/**
	 * 参与活动商品
	 */
	@TableField(value = "ip_address")
	private String ipAddress;

	@TableField(value = "participate_goods")
	private String participateGoods;

	/**
	 * 参与活动渠道
	 */
	@TableField(value = "participate_channel")
	private String participateChannel;

	/**
	 * 参与活动时间
	 */
	@TableField(value = "participate_time")
	private Date participateTime;

	/**
	 * 是否中奖 1 是 0 否
	 */
	@TableField(value = "is_hit")
	private Integer isHit;

	/**
	 * 是否订购 1 是 0 否
	 */
	@TableField(value = "is_order")
	private String isOrder;

	public ActivityDataRecordJoin convert() {
		ActivityDataRecordJoin data = new ActivityDataRecordJoin();
		data.setJoinId(id);
		data.setActivityUuid(activityUuid);
		data.setActivityName(activityName);
		data.setActivityType(activityType);
		data.setActivityTypeName(ActivityTypeEnum.reslove(activityType).getMessage());
		data.setPhone(participatePhone);
		data.setIp(ipAddress);
		data.setIsHit(isHit);
		data.setJoinTime(participateTime);
		data.setCreateTime(new Date());
		return data;
	}
}
