package com.platomix.activity.clickhouse.model;

import java.io.Serializable;
import java.math.BigDecimal;
import java.time.LocalDateTime;
import java.time.ZoneId;
import java.util.Date;

import com.baomidou.mybatisplus.annotation.IdType;
import com.baomidou.mybatisplus.annotation.TableField;
import com.baomidou.mybatisplus.annotation.TableId;
import com.baomidou.mybatisplus.annotation.TableName;
import com.fasterxml.jackson.annotation.JsonFormat;
import com.platomix.activity.enums.ActivityTypeEnum;
import com.platomix.activity.enums.StatusEnum;
import com.platomix.activity.enums.ThirdEnum;
import com.platomix.activity.kit.SnowKit;

import cn.hutool.core.date.LocalDateTimeUtil;
import io.swagger.annotations.ApiModelProperty;
import lombok.Data;

/**
 * 
 * Desc: 活动
 * <p>
 * Modify:
 * <p>
 * Author: William Lynn
 * <p>
 * Create Date：Aug 18, 2021 5:50:15 PM
 *
 */
@Data
@TableName("DIC_BLADE_ACTIVITY")
public class BladeActivity implements Serializable {

	private static final long serialVersionUID = 1L;

	@TableId(value = "id", type = IdType.AUTO)
	@ApiModelProperty(value = "主键ID")
	private Long id;

	@TableField(value = "title")
	@ApiModelProperty(value = "活动主题")
	private String title;

	@TableField(value = "type")
	@ApiModelProperty(value = "活动形式")
	private Integer type;

	@TableField(value = "activity_start_time")
	@ApiModelProperty(value = "活动时间")
	@JsonFormat(pattern = "yyyy-MM-dd")
	private Date activityStartTime;

	@TableField(value = "activity_end_time")
	@ApiModelProperty(value = "活动结束时间")
	@JsonFormat(pattern = "yyyy-MM-dd")
	private Date activityEndTime;

	@TableField(value = "cash_deadline")
	@ApiModelProperty(value = "兑奖截止时间")
	private Date cashDeadline;

	@TableField(value = "activity_uuid")
	@ApiModelProperty(value = "活动UUID")
	private String activityUuid;

	@TableField(value = "activity_province")
	@ApiModelProperty(value = "活动地区 省份")
	private String activityProvince;

	@TableField(value = "activity_city")
	@ApiModelProperty(value = "活动地区 市/区")
	private String activityCity;

	@TableField(value = "activity_region")
	@ApiModelProperty(value = "活动地区")
	private String activityRegion;

	@TableField(value = "activity_totalfee")
	@ApiModelProperty(value = "活动总成本")
	private BigDecimal activityTotalfee;

	@JsonFormat(pattern = "yyyy-MM-dd HH:mm:ss")
	@TableField(value = "create_time")
	@ApiModelProperty(value = "创建时间")
	private Date createTime;

	@JsonFormat(pattern = "yyyy-MM-dd HH:mm:ss")
	@TableField(value = "update_time")
	@ApiModelProperty(value = "修改时间")
	private Date updateTime;

	@ApiModelProperty(value = "状态")
	private Integer status;

	@TableField(value = "is_deleted")
	@ApiModelProperty(value = "是否已删除")
	private Integer isDeleted;

	@TableField(value = "is_third")
	@ApiModelProperty(value = "是否第三方活动")
	private Integer isThird;

	public ActivityDataBaseInfo convert() {
		ActivityDataBaseInfo info = new ActivityDataBaseInfo();
		info.setId(SnowKit.next());
		info.setActivityId(id);
		info.setActivityUuid(activityUuid);
		info.setActivityName(title);
		info.setActivityType(type);
		info.setActivityTypeName(ActivityTypeEnum.reslove(type).getMessage());
		info.setStartTime(activityStartTime);
		info.setEndTime(activityEndTime);
		info.setCashDeadline(cashDeadline);
		info.setStatus(status);
		info.setStatusName(StatusEnum.reslove(status).getMessage());
		info.setIsThird(isThird);
		info.setIsThirdName(ThirdEnum.reslove(isThird).getMessage());
		info.setCreateTime(new Date());
		
		LocalDateTime deadLine = null;
		if (cashDeadline != null) {
			deadLine = LocalDateTimeUtil.of(cashDeadline).plusMonths(2);
		} else {
			deadLine = LocalDateTimeUtil.of(activityEndTime).plusMonths(2);
		}
		
		if (deadLine != null) {
			Date date = Date.from( deadLine.atZone( ZoneId.systemDefault()).toInstant());
			info.setDeadline(date);
		}
		return info;
	}
}
