package com.platomix.activity.clickhouse.service;

import java.time.LocalDateTime;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import com.baomidou.dynamic.datasource.annotation.DS;
import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.core.toolkit.Wrappers;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.platomix.activity.clickhouse.mapper.ActivityDataMapper;
import com.platomix.activity.clickhouse.model.ActivityData;
import com.platomix.activity.enums.DateTypeEnum;
import com.platomix.activity.kit.TimeKit;

/**
 * Desc:
 * <p>
 * Modify:
 * <p>
 * Author: William Lynn
 * <p>
 * Create Date：Aug 18, 2021 4:23:35 PM
 * 
 */
@DS("clickhouse")
@Service
public class ActivityDataService extends ServiceImpl<ActivityDataMapper, ActivityData> {

	@Autowired
	private TimeKit timeKit;

	@Autowired
	private ActivityDataBaseInfoService activityService;

	public void clearMidTable() {
		baseMapper.truncateJoin();
		baseMapper.truncateExchange();
	}

	public void buildJoinAbout(String field, int startTime, int endTime) {
		baseMapper.buildJoinAbout(field, startTime, endTime);
	}

	public void buildExchangeAbout(String field, int startTime, int endTime) {
		baseMapper.buildExchangeAbout(field, startTime, endTime);
	}

	public void deletePartitionByKey(int key) {
		baseMapper.deletePartitionByKey(key);
	}

	public void deletePartitionByRange(int keyStart, int keyEnd) {
		baseMapper.deletePartitionByRange(keyStart, keyEnd);
	}

	/**
	 * 获取当前已处理的最大年月，例如：202110
	 * 
	 * @return
	 */
	public LocalDateTime getLastDateMonth() {
		LocalDateTime date = null;
		LambdaQueryWrapper<ActivityData> queryWrapper = Wrappers.<ActivityData>query().lambda()
				.orderByDesc(ActivityData::getTheDValue).last("LIMIT 0,1");
		ActivityData data = baseMapper.selectOne(queryWrapper);
		if (data == null) {
			date = activityService.getOldestActivityStartTime();
		} else {
			date = timeKit.getDimensionStartTime(DateTypeEnum.valueOf(data.getDType()), data.getTheDValue());
		}

		return date;
	}

	public void merge() {
		baseMapper.merge();
	}

}
