package com.platomix.activity.kit;

import java.io.Serializable;
import java.lang.reflect.Type;
import java.util.ArrayList;
import java.util.List;

import com.google.gson.Gson;
import com.google.gson.JsonArray;
import com.google.gson.JsonElement;
import com.google.gson.JsonParser;
import com.platomix.activity.common.Ret;
import com.platomix.activity.common.PageData;

import ikidou.reflect.TypeBuilder;

/**
 * Depiction:Json解析工具
 * <p>
 * Modify:
 * <p>
 * Author: Kevin Lynn
 * <p>
 * Create Date：2018年4月23日 下午6:48:25
 */
public class JsonKit {

    public JsonKit() {
    }

    /**
     * Json数据解析
     *
     * @param <T>
     * @param json json源串
     * @return List<T>
     */
    public static <T> List<T> parseArray(String json, Class<T> type) {
        List<T> list = new ArrayList<>();
        try {
            Gson gson = new Gson();
            JsonArray jsonarray = JsonParser.parseString(json).getAsJsonArray();
            for (JsonElement element : jsonarray) {
                list.add(gson.fromJson(element, type));
            }
        } catch (Exception e) {
            System.err.println("JsonKit parseArray()" + e.toString());
            System.err.println();
        }
        return list;
    }

    /**
     * Json数据解析
     *
     * @param json json源串
     * @param type 存储json的实体类
     * @return 相应的实体类对象
     */
    public static <T> T parseJson(String json, Type type) {
        try {
            Gson gson = new Gson();
            return gson.fromJson(json, type);
        } catch (Exception e) {
            System.err.println(type.getClass().getSimpleName() + " parseJson()" + e.toString());
            System.err.println();
        }
        return null;
    }

    /**
     * 解析数组数据
     *
     * @param json
     * @param clazz
     * @param <T>
     * @return
     */
    public static <T> Ret<List<T>> parseList(String json, Class<T> clazz) {
        try {
            Type type = TypeBuilder
                    .newInstance(Ret.class)
                    .beginSubType(List.class)
                    .addTypeParam(clazz)
                    .endSubType()
                    .build();
            return new Gson().fromJson(json, type);
        } catch (Exception e) {
        }

        return null;
    }

    /**
     * 解析对象数据
     *
     * @param json
     * @param clazz
     * @param <T>
     * @return
     */
    public static <T> Ret<T> parseObject(String json, Class<T> clazz) {
        try {
            Type type = TypeBuilder
                    .newInstance(Ret.class)
                    .addTypeParam(clazz)
                    .build();
            return new Gson().fromJson(json, type);
        } catch (Exception e) {
        }

        return null;
    }

    /**
     * 解析分页数据
     *
     * @param json
     * @param clazz
     * @param <T>
     * @return
     */
    public static <T extends Serializable> Ret<PageData<T>> parsePage(String json, Class<T> clazz) {
        try {
            Type type = TypeBuilder
                    .newInstance(Ret.class)
                    .beginSubType(PageData.class)
                    .addTypeParam(clazz)
                    .endSubType()
                    .build();
            return new Gson().fromJson(json, type);
        } catch (Exception e) {
        }

        return null;
    }


    public static String toJson(Object obj) {
        String json = "";
        if (obj != null) {
            try {
                json = new Gson().toJson(obj);

            } catch (Exception e) {
            }
        }
        return json;
    }
}
