package com.platomix.activity.kit;

import java.time.LocalDateTime;

import org.springframework.stereotype.Component;

import com.platomix.activity.enums.DateTypeEnum;

import cn.hutool.core.date.DatePattern;
import cn.hutool.core.date.LocalDateTimeUtil;
import cn.hutool.core.util.NumberUtil;

/**
 * Desc:根据维度计算指标的时间范围
 * <p>
 * Modify:
 * <p>
 * Author: William Lynn
 * <p>
 * Create Date：Sep 11, 2021 4:31:03 PM
 * 
 */
@Component
public class TimeKit {

	final String monthPattern = "yyyyMM";
	final String dayPattern = "yyyyMMdd";
	final String hourPattern = "yyyyMMddHH";

//	public Date of(LocalDateTime localDateTime) {
//		Date date = Date.from(localDateTime.atZone(ZoneId.systemDefault()).toInstant());
//		return date;
//	}
	
	public String of(LocalDateTime localDateTime) {
		return LocalDateTimeUtil.format(localDateTime, DatePattern.NORM_DATETIME_FORMATTER);
	}

	/**
	 * 根据日期类型和时间，计算日期纬度值
	 * 
	 * @param type
	 * @param time
	 * @return
	 */
	public int getDateValue(DateTypeEnum type, LocalDateTime time) {
		String dateValue = "0";
		if (type == DateTypeEnum.MONTH) {
			dateValue = LocalDateTimeUtil.format(time, monthPattern);
		} else if (type == DateTypeEnum.DAY) {
			dateValue = LocalDateTimeUtil.format(time, dayPattern);
		} else if (type == DateTypeEnum.HOUR) {
			dateValue = LocalDateTimeUtil.format(time, hourPattern);
		}

		int value = NumberUtil.parseInt(dateValue);
		return value;
	}

	public LocalDateTime getDimensionStartTime(DateTypeEnum type, int dateValue) {
		if (type == DateTypeEnum.MONTH) {
			// 月维度
			return LocalDateTimeUtil.parse(String.valueOf(dateValue), monthPattern);
		} else if (type == DateTypeEnum.DAY) {
			// 日维度
			return LocalDateTimeUtil.parse(String.valueOf(dateValue), dayPattern);
		}
		if (type == DateTypeEnum.HOUR) {
			// 小时维度
			return LocalDateTimeUtil.parse(String.valueOf(dateValue), hourPattern);
		}
		return null;
	}

	public LocalDateTime getDimensionEndTime(DateTypeEnum type, int dateValue) {
		LocalDateTime date = getDimensionStartTime(type, dateValue);
		if (type == DateTypeEnum.MONTH) {
			// 月维度
			return date.plusMonths(1).minusSeconds(1);
		} else if (type == DateTypeEnum.DAY) {
			// 日维度
			return date.plusDays(1).minusSeconds(1);
		}
		if (type == DateTypeEnum.HOUR) {
			// 小时维度
			return date.plusHours(1).minusSeconds(1);
		}

		return null;
	}

	/**
	 * 根据维度时间和活动开始结束时间，检测是否为有效时间内
	 * 
	 * @param type
	 * @param dateValue
	 * @param startTimeActivity
	 * @param endTimeActivity
	 * @return
	 */
	public boolean checkDimensionTimeValid(DateTypeEnum type, int dateValue, LocalDateTime startTimeActivity,
			LocalDateTime endTimeActivity) {
		LocalDateTime startTimeDimension = getDimensionStartTime(type, dateValue);
		LocalDateTime endTimeDimension = getDimensionEndTime(type, dateValue);

		int dateValueStart = getDateValue(type, startTimeDimension);
		int dateValueEnd = getDateValue(type, endTimeDimension);

		if (dateValue >= dateValueStart && dateValue <= dateValueEnd) {
			return true;
		}

		return false;
	}
}
