package com.platomix.activity.job;

import java.math.BigDecimal;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.List;
import java.util.concurrent.ExecutorService;
import java.util.concurrent.Executors;
import java.util.concurrent.TimeUnit;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;

import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.core.toolkit.Wrappers;
import com.platomix.activity.clickhouse.model.ActivityDataBaseInfo;
import com.platomix.activity.clickhouse.model.BladeActivity;
import com.platomix.activity.clickhouse.model.BladeActivityChannel;
import com.platomix.activity.clickhouse.model.BladeActivityPrize;
import com.platomix.activity.clickhouse.model.BladeActivityProduct;
import com.platomix.activity.clickhouse.service.ActivityDataBaseInfoService;
import com.platomix.activity.clickhouse.service.BladeActivityChannelService;
import com.platomix.activity.clickhouse.service.BladeActivityCostService;
import com.platomix.activity.clickhouse.service.BladeActivityPrizeService;
import com.platomix.activity.clickhouse.service.BladeActivityProductService;
import com.platomix.activity.clickhouse.service.BladeActivityService;
import com.platomix.activity.kit.JsonKit;

import cn.hutool.core.collection.CollUtil;
import lombok.extern.slf4j.Slf4j;

/**
 * Desc:
 * <p>
 * Modify:
 * <p>
 * Author: William Lynn
 * <p>
 * Create Date：Oct 18, 2021 3:49:58 PM
 * 
 */
@Slf4j
@Component
public class ActivityInfoTask extends AbsTask {

	@Autowired
	private ActivityDataBaseInfoService infoService;

	@Autowired
	private BladeActivityService activityService;

	@Autowired
	private BladeActivityChannelService channelService;

	@Autowired
	private BladeActivityCostService costService;

	@Autowired
	private BladeActivityPrizeService prizeService;

	@Autowired
	private BladeActivityProductService productService;

	public void start() {
		int threadNum = THREAD_NUM;
		final ExecutorService threadExecutor = Executors.newFixedThreadPool(threadNum);
		for (int i = 0; i < threadNum; i++) {
			threadExecutor.execute(new TaskThread(i, threadNum));
		}
	}

	class TaskThread extends Thread {
		private List<ActivityDataBaseInfo> dataList;

		private long mod;
		private int threadNum;

		public TaskThread(long mod, int threadNum) {
			this.mod = mod;
			this.threadNum = threadNum;
		}

		@Override
		public void run() {
			dataList = new ArrayList<>();
			while (true) {
				log.info("活动数据清洗【" + (mod + 1) + "】已完成");
				long lastId = getMaxId(mod, threadNum);
				LambdaQueryWrapper<BladeActivity> queryWrapper = Wrappers.<BladeActivity>query().lambda()
						.gt(BladeActivity::getId, lastId).orderByAsc(BladeActivity::getId).last("limit 0," + LIMIT)
						.apply("id%{0}={1}", threadNum, mod);
				List<BladeActivity> list = activityService.list(queryWrapper);
				if (CollUtil.isNotEmpty(list)) {
					for (BladeActivity item : list) {
						ActivityDataBaseInfo info = item.convert();
						dataList.add(handle(info, item));

						if (dataList.size() >= BATCH_SIZE) {
							infoService.saveBatch(dataList);
							dataList.clear();
							log.info("活动数据清洗任务【" + (mod + 1) + "】- 插入数据");
						}
					}
				}

				if (!dataList.isEmpty()) {
					infoService.saveBatch(dataList);
					dataList.clear();
				}

				if (list == null || list.size() < LIMIT) {
					// 查询完成
					log.info("活动数据清洗【" + (mod + 1) + "】已完成，5分钟后继续查询");
					try {
						// 5秒后继续查询
						TimeUnit.MINUTES.sleep(5);
					} catch (InterruptedException e) {
						log.info("活动数据清洗任务【" + (mod + 1) + "】异常 ，错误信息：" + e.getMessage());
					}
				}
			}
		}
	}

	private ActivityDataBaseInfo handle(ActivityDataBaseInfo data, BladeActivity item) {
		List<BladeActivityChannel> channels = channelService.get(item.getId());
		List<BladeActivityProduct> products = productService.get(item.getId());
		List<BladeActivityPrize> prizes = prizeService.get(item.getId());

		List<String> spidList = new ArrayList<>();

		float costTotalOther = costService.getCostTotalOther(item.getId());
		float costTotalChannel = 0;
		float costTotalProduct = 0;
		float costTotalPrize = 0;

		if (CollUtil.isNotEmpty(channels)) {
			for (BladeActivityChannel channel : channels) {
				costTotalChannel += channel.getCostPrice().floatValue();
			}
		}

		if (CollUtil.isNotEmpty(products)) {
			for (BladeActivityProduct product : products) {
				costTotalProduct += product.getCostPrice().floatValue();
				spidList.add(product.getSpid());
			}
		}

		if (CollUtil.isNotEmpty(prizes)) {
			for (BladeActivityPrize prize : prizes) {
				costTotalPrize += prize.getCostPrice().floatValue();
			}
		}
		data.setCostTotalChannel(BigDecimal.valueOf(costTotalChannel));
		data.setCostTotalOther(BigDecimal.valueOf(costTotalOther));
		data.setCostTotalActivity(item.getActivityTotalfee());
		data.setCostTotalPrize(BigDecimal.valueOf(costTotalPrize));
		data.setCostTotalProduct(BigDecimal.valueOf(costTotalProduct));
		if (CollUtil.isNotEmpty(spidList)) {
			data.setSpids(spidList.stream().toArray(String[]::new));
		}
		return data;
	}

	private Long getMaxId(long mod, int threadNum) {
		LambdaQueryWrapper<ActivityDataBaseInfo> queryWrapper = Wrappers.<ActivityDataBaseInfo>query().lambda()
				.orderByDesc(ActivityDataBaseInfo::getActivityId).last("limit 0,1")
				.apply("ACTIVITY_ID%{0}={1}", threadNum, mod);
		ActivityDataBaseInfo data = infoService.getOne(queryWrapper);
		return data != null ? data.getActivityId() : 0;
	}

	public static void main(String[] args) {
		List<String> list = Arrays.asList("1", "2");
		System.out.println(JsonKit.toJson(list).replaceAll("\"", "'"));
	}
}
