package com.platomix.activity.job;

import java.util.ArrayList;
import java.util.List;
import java.util.concurrent.ExecutorService;
import java.util.concurrent.Executors;
import java.util.concurrent.TimeUnit;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;

import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.core.toolkit.Wrappers;
import com.platomix.activity.clickhouse.model.ActivityDataRecordExchange;
import com.platomix.activity.clickhouse.model.ActivityMemberExchangeRecord;
import com.platomix.activity.clickhouse.model.CallerAreaCode;
import com.platomix.activity.clickhouse.service.ActivityDataRecordExchangeService;
import com.platomix.activity.clickhouse.service.ActivityMemberExchangeRecordService;
import com.platomix.activity.enums.DateTypeEnum;

import cn.hutool.core.collection.CollUtil;
import cn.hutool.core.date.LocalDateTimeUtil;
import lombok.extern.slf4j.Slf4j;

/**
 * Desc:
 * <p>
 * Modify:
 * <p>
 * Author: William Lynn
 * <p>
 * Create Date：Oct 18, 2021 3:49:58 PM
 * 
 */
@Slf4j
@Component
public class ExchangeRecordTask extends AbsTask {

	@Autowired
	private ActivityMemberExchangeRecordService cashService;

	@Autowired
	private ActivityDataRecordExchangeService exchangeService;

	public void start() {
		int threadNum = THREAD_NUM;
		final ExecutorService threadExecutor = Executors.newFixedThreadPool(threadNum);
		for (int i = 0; i < threadNum; i++) {
			threadExecutor.execute(new TaskThread(i, threadNum));
		}
	}

	class TaskThread extends Thread {
		private List<ActivityDataRecordExchange> dataList;

		private long mod;
		private int threadNum;

		public TaskThread(long mod, int threadNum) {
			this.mod = mod;
			this.threadNum = threadNum;
		}

		@Override
		public void run() {
			dataList = new ArrayList<>();
			while (true) {
				log.info("兑奖记录数据清洗任务【" + (mod + 1) + "】已启动");
				long lastId = getMaxId(mod, threadNum);
				LambdaQueryWrapper<ActivityMemberExchangeRecord> queryWrapper = Wrappers
						.<ActivityMemberExchangeRecord>query().lambda().gt(ActivityMemberExchangeRecord::getId, lastId)
						.orderByAsc(ActivityMemberExchangeRecord::getId).last("limit 0," + LIMIT)
						.apply("id%{0}={1} ", threadNum, mod);
				List<ActivityMemberExchangeRecord> list = cashService.list(queryWrapper);
				if (CollUtil.isNotEmpty(list)) {
					for (ActivityMemberExchangeRecord item : list) {
						ActivityDataRecordExchange data = item.convert();
						data = handle(data, item);
						if (data != null) {
							dataList.add(data);

							if (dataList.size() >= BATCH_SIZE) {
								exchangeService.saveBatch(dataList);
								dataList.clear();
								log.info("兑奖记录数据清洗任务【" + (mod + 1) + "】- 插入数据");
							}
						}
					}
				}

				if (!dataList.isEmpty()) {
					exchangeService.saveBatch(dataList);
					dataList.clear();
				}

				if (list == null || list.size() < LIMIT) {
					// 查询完成
					log.info("兑奖记录数据清洗任务【" + (mod + 1) + "】已完成，5分钟后继续查询");
					try {
						TimeUnit.MINUTES.sleep(5);
					} catch (InterruptedException e) {
						log.info("兑奖记录数据清洗任务【" + (mod + 1) + "】异常 ，错误信息：" + e.getMessage());
					}
				}
			}
		}
	}

	private ActivityDataRecordExchange handle(ActivityDataRecordExchange data, ActivityMemberExchangeRecord item) {
		CallerAreaCode area = getArea(data.getPhone());
		if (area == null) {
			return null;
		}

		data.setProvince(area.getProvince());
		data.setCity(area.getArea());

		data.setDateValueMonth(timeKit.getDateValue(DateTypeEnum.MONTH, LocalDateTimeUtil.of(data.getExchangeTime())));
		data.setDateValueDay(timeKit.getDateValue(DateTypeEnum.DAY, LocalDateTimeUtil.of(data.getExchangeTime())));
		data.setDateValueHour(timeKit.getDateValue(DateTypeEnum.HOUR, LocalDateTimeUtil.of(data.getExchangeTime())));

		return data;
	}

	private Long getMaxId(long mod, int threadNum) {
		LambdaQueryWrapper<ActivityDataRecordExchange> queryWrapper = Wrappers.<ActivityDataRecordExchange>query()
				.lambda().orderByDesc(ActivityDataRecordExchange::getExchangeId).last("limit 0,1")
				.apply("EXCHANGE_ID%{0}={1}", threadNum, mod);
		ActivityDataRecordExchange data = exchangeService.getOne(queryWrapper);
		return data != null ? data.getExchangeId() : 0;
	}
}
