package com.platomix.activity.job;

import java.util.ArrayList;
import java.util.List;
import java.util.concurrent.ExecutorService;
import java.util.concurrent.Executors;
import java.util.concurrent.TimeUnit;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;

import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.core.toolkit.Wrappers;
import com.platomix.activity.clickhouse.model.ActivityDataRecordJoin;
import com.platomix.activity.clickhouse.model.ActivityParticipateRecord;
import com.platomix.activity.clickhouse.model.BladeActivityChannel;
import com.platomix.activity.clickhouse.model.CallerAreaCode;
import com.platomix.activity.clickhouse.service.ActivityDataRecordJoinService;
import com.platomix.activity.clickhouse.service.ActivityParticipateRecordService;
import com.platomix.activity.clickhouse.service.BladeActivityChannelService;
import com.platomix.activity.enums.DateTypeEnum;

import cn.hutool.core.collection.CollUtil;
import cn.hutool.core.date.LocalDateTimeUtil;
import cn.hutool.core.util.StrUtil;
import lombok.extern.slf4j.Slf4j;

/**
 * Desc:
 * <p>
 * Modify:
 * <p>
 * Author: William Lynn
 * <p>
 * Create Date：Oct 18, 2021 3:49:58 PM
 * 
 */
@Slf4j
@Component
public class JoinRecordTask extends AbsTask {

	@Autowired
	private ActivityParticipateRecordService participateService;

	@Autowired
	private ActivityDataRecordJoinService joinService;

	@Autowired
	private BladeActivityChannelService channelService;

	public void start() {
		int threadNum = THREAD_NUM;
		final ExecutorService threadExecutor = Executors.newFixedThreadPool(threadNum);
		for (int i = 0; i < threadNum; i++) {
			threadExecutor.execute(new TaskThread(i, threadNum));
		}
	}

	class TaskThread extends Thread {
		private List<ActivityDataRecordJoin> dataList;

		private long mod;
		private int threadNum;

		public TaskThread(long mod, int threadNum) {
			this.mod = mod;
			this.threadNum = threadNum;
		}

		@Override
		public void run() {
			dataList = new ArrayList<>();
			while (true) {
				log.info("参与记录数据清洗任务【" + (mod + 1) + "】已启动");
				long lastId = getMaxId(mod, threadNum);
				LambdaQueryWrapper<ActivityParticipateRecord> queryWrapper = Wrappers.<ActivityParticipateRecord>query()
						.lambda().gt(ActivityParticipateRecord::getId, lastId)
						.orderByAsc(ActivityParticipateRecord::getId).last("limit 0," + LIMIT)
						.apply("id%{0}={1}", threadNum, mod);
				List<ActivityParticipateRecord> list = participateService.list(queryWrapper);
				if (CollUtil.isNotEmpty(list)) {
					for (ActivityParticipateRecord item : list) {
						ActivityDataRecordJoin data = item.convert();
						data = handle(data, item);
						if (data != null) {
							dataList.add(data);

							if (dataList.size() >= BATCH_SIZE) {
								joinService.saveBatch(dataList);
								dataList.clear();
								log.info("参与记录数据清洗任务【" + (mod + 1) + "】- 插入数据");
							}
						}
					}
				}

				if (!dataList.isEmpty()) {
					joinService.saveBatch(dataList);
					dataList.clear();
				}

				if (list == null || list.size() < LIMIT) {
					// 查询完成
					log.info("参与记录数据清洗任务【" + (mod + 1) + "】已完成，5分钟后继续查询");
					try {
						TimeUnit.SECONDS.sleep(5);
					} catch (InterruptedException e) {
						log.info("参与记录数据清洗任务【" + (mod + 1) + "】异常 ，错误信息：" + e.getMessage());
					}
				}
			}
		}
	}

	private ActivityDataRecordJoin handle(ActivityDataRecordJoin data, ActivityParticipateRecord item) {
		String channelId = item.getParticipateChannel();
		if (StrUtil.isBlankIfStr(channelId)) {
			channelId = "166887";
		}

		BladeActivityChannel channel = getChannel(channelId);

		data.setChannelId(channelId);
		data.setChannelIdOrder(channel.getOrderId());
		data.setChannelType(channel.getChannelType());
		data.setChannelName(channel.getName());

		CallerAreaCode area = getArea(data.getPhone());
		if (area == null) {
			return null;
		}

		data.setProvince(area.getProvince());
		data.setCity(area.getArea());

		data.setDateValueMonth(timeKit.getDateValue(DateTypeEnum.MONTH, LocalDateTimeUtil.of(data.getJoinTime())));
		data.setDateValueDay(timeKit.getDateValue(DateTypeEnum.DAY, LocalDateTimeUtil.of(data.getJoinTime())));
		data.setDateValueHour(timeKit.getDateValue(DateTypeEnum.HOUR, LocalDateTimeUtil.of(data.getJoinTime())));

		return data;
	}

	private BladeActivityChannel getChannel(String channelId) {
		LambdaQueryWrapper<BladeActivityChannel> queryWrapper = Wrappers.<BladeActivityChannel>query().lambda()
				.eq(BladeActivityChannel::getChannelId, channelId).orderByAsc(BladeActivityChannel::getId)
				.last("limit 0,1");
		return channelService.getOne(queryWrapper);
	}

	private Long getMaxId(long mod, int threadNum) {
		LambdaQueryWrapper<ActivityDataRecordJoin> queryWrapper = Wrappers.<ActivityDataRecordJoin>query().lambda()
				.orderByDesc(ActivityDataRecordJoin::getJoinId).last("limit 0,1")
				.apply("JOIN_ID%{0}={1}", threadNum, mod);
		ActivityDataRecordJoin data = joinService.getOne(queryWrapper);
		return data != null ? data.getJoinId() : 0;
	}

}
