package com.platomix.activity.config;

import org.springframework.beans.factory.annotation.Value;
import org.springframework.context.annotation.Bean;
import org.springframework.context.annotation.Configuration;

import com.jfinal.template.Engine;
import com.jfinal.template.ext.spring.JFinalViewResolver;

/**
 * Desc:
 * <p>
 * Modify:
 * <p>
 * Author: William Lynn
 * <p>
 * Create Date：Sep 4, 2021 6:11:27 PM
 * 
 */
@Configuration
public class EnjoyConfig {

	@Value("${server.servlet.context-path}")
	private String contextPath;

	@Bean(name = "jfinalViewResolver")
	public JFinalViewResolver jFinalViewResolver() {

		// 创建用于整合 spring boot 的 ViewResolver 扩展对象
		JFinalViewResolver jfr = new JFinalViewResolver();

		// 对 spring boot 进行配置
		jfr.setSuffix(".html");
		jfr.setContentType("text/html;charset=UTF-8");
		jfr.setOrder(0);

		// 设置在模板中可通过 #(session.value) 访问 session 中的数据
		jfr.setSessionInView(true);

		// 设置静态文件域名[正式环境不需要contextPath，否则需要添加]
//		jfr.addSharedObject("contextPath", contextPath);

		// 获取 engine 对象，对 enjoy 模板引擎进行配置，配置方式与前面章节完全一样
		Engine engine = JFinalViewResolver.engine;

		// 热加载配置能对后续配置产生影响，需要放在最前面
		engine.setDevMode(true);

		// 使用 ClassPathSourceFactory 从 class path 与 jar 包中加载模板文件
		engine.setToClassPathSourceFactory();

		// 在使用 ClassPathSourceFactory 时要使用 setBaseTemplatePath
		// 代替 jfr.setPrefix("/view/")
		engine.setBaseTemplatePath("/views/");

		return jfr;
	}

}
