package com.platomix.activity.controller;

import java.time.LocalDateTime;
import java.time.format.DateTimeFormatter;

import javax.servlet.http.HttpSession;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Controller;
import org.springframework.ui.Model;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.ResponseBody;

import com.jfinal.kit.Kv;
import com.platomix.activity.clickhouse.service.ActivityDataBaseInfoService;
import com.platomix.activity.clickhouse.service.ActivityDataService;
import com.platomix.activity.common.Ret;
import com.platomix.activity.enums.DateTypeEnum;
import com.platomix.activity.kit.TimeKit;

import cn.hutool.core.date.LocalDateTimeUtil;
import lombok.extern.slf4j.Slf4j;

/**
 * Desc: 首页控制器
 * <p>
 * Modify:
 * <p>
 * Author: William Lynn
 * <p>
 * Create Date：Sep 4, 2021 5:57:49 PM
 * 
 */
@Slf4j
@Controller
@RequestMapping("/index")
public class IndexController {
	@Autowired
	private TimeKit timeKit;

	@Autowired
	private ActivityDataService dataService;

	@Autowired
	private ActivityDataBaseInfoService activityService;

	@GetMapping("/")
	public String index(Model model, HttpSession session) {
		return "index";
	}

	@GetMapping("/delete")
	@ResponseBody
	public Ret<Kv> delete(int key) {
		dataService.deletePartitionByKey(key);
		return Ret.success("分区「" + key + "」删除成功");
	}

	@GetMapping("/delete/range")
	@ResponseBody
	public Ret<Kv> deleteRange(int keyStart, int keyEnd) {
		dataService.deletePartitionByRange(keyStart, keyEnd);
		return Ret.success("分区「" + keyStart + " - " + keyEnd + "」删除成功");
	}

	@GetMapping("/perform")
	@ResponseBody
	public Ret<Kv> perform(String start, String end) {
		long s = System.currentTimeMillis();
		LocalDateTime timeStart = null;
		LocalDateTime timeEnd = null;
		try {
			timeStart = LocalDateTimeUtil.parse(start + "01", DateTimeFormatter.BASIC_ISO_DATE);
			timeEnd = LocalDateTimeUtil.parse(end + "01", DateTimeFormatter.BASIC_ISO_DATE);
		} catch (Exception e) {
			log.info("perform fail, the exception info is " + e.getMessage());
		}

		if (timeStart == null || timeEnd == null) {
			return Ret.error("操作失败，参数date格式为：yyyyMM,请检查");
		}

		// 月初第一个小时
		timeStart = LocalDateTimeUtil.beginOfDay(timeStart);
		// 月末最后一个小时
		timeEnd = LocalDateTimeUtil.endOfDay(timeEnd.plusMonths(1).minusDays(1));

		// 清空中间表
//		dataService.clearMidTable();
		// 删除该月份已生成的指标
//		dataService.deletePartitionByKey(Integer.parseInt(date));

		int dateStart = 0;
		int dateEnd = 0;
		DateTypeEnum[] types = DateTypeEnum.values();
		for (DateTypeEnum type : types) {
			dateStart = timeKit.getDateValue(type, timeStart);
			dateEnd = timeKit.getDateValue(type, timeEnd);
			dataService.buildJoinAbout(type.name(), dateStart, dateEnd);
			dataService.buildExchangeAbout(type.name(), dateStart, dateEnd);
		}

		dataService.merge();

		long e = System.currentTimeMillis();
		return Ret.success("操作成功", Kv.by("time", (e - s) / 1000.0 + "秒").set("start", start).set("end", end));
	}

}
